Stan Math Library: stan/math/prim/scal/prob/pareto_type_2_ccdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

pareto_type_2_ccdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_PARETO_TYPE_2_CCDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_PARETO_TYPE_2_CCDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
5 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
6 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
7 #include <stan/math/prim/scal/err/check_finite.hpp>
8 #include <stan/math/prim/scal/err/check_greater_or_equal.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
12 #include <stan/math/prim/scal/fun/constants.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/fun/log1m.hpp>
15 #include <stan/math/prim/scal/meta/contains_nonconstant_struct.hpp>
16 #include <stan/math/prim/scal/meta/VectorView.hpp>
17 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
18 #include <boost/random/variate_generator.hpp>
19 #include <cmath>
20
21 namespace stan {
22 namespace math {
23
24 template <typename T_y, typename T_loc, typename T_scale, typename T_shape>
25 typename return_type<T_y, T_loc, T_scale, T_shape>::type
26 pareto_type_2_ccdf_log(const T_y& y, const T_loc& mu,
27 const T_scale& lambda, const T_shape& alpha) {
28 typedef
29 typename stan::partials_return_type<T_y, T_loc, T_scale, T_shape>::type
30 T_partials_return;
31
32 // Check sizes
33 // Size checks
34 if ( !( stan::length(y)
35 && stan::length(mu)
36 && stan::length(lambda)
37 && stan::length(alpha) ) )
38 return 0.0;
39
40 // Check errors
41 static const char* function("stan::math::pareto_type_2_ccdf_log");
42
43 using stan::math::check_greater_or_equal;
44 using stan::math::check_positive_finite;
45 using stan::math::check_not_nan;
46 using stan::math::check_greater_or_equal;
47 using stan::math::check_consistent_sizes;
48 using stan::math::check_nonnegative;
49 using stan::math::value_of;
50 using std::log;
51
52 T_partials_return P(0.0);
53
54 check_greater_or_equal(function, "Random variable", y, mu);
55 check_not_nan(function, "Random variable", y);
56 check_nonnegative(function, "Random variable", y);
57 check_positive_finite(function, "Scale parameter", lambda);
58 check_positive_finite(function, "Shape parameter", alpha);
59 check_consistent_sizes(function,
60 "Random variable", y,
61 "Scale parameter", lambda,
62 "Shape parameter", alpha);
63
64 // Wrap arguments in vectors
65 VectorView<const T_y> y_vec(y);
66 VectorView<const T_loc> mu_vec(mu);
67 VectorView<const T_scale> lambda_vec(lambda);
68 VectorView<const T_shape> alpha_vec(alpha);
69 size_t N = max_size(y, mu, lambda, alpha);
70
71 OperandsAndPartials<T_y, T_loc, T_scale, T_shape>
72 operands_and_partials(y, mu, lambda, alpha);
73
74 VectorBuilder<true, T_partials_return,
75 T_y, T_loc, T_scale, T_shape>
76 ccdf_log(N);
77
78 VectorBuilder<contains_nonconstant_struct<T_y, T_loc, T_scale,
79 T_shape>::value,
80 T_partials_return, T_y, T_loc, T_scale, T_shape>
81 a_over_lambda_plus_y(N);
82
83 VectorBuilder<!is_constant_struct<T_shape>::value,
84 T_partials_return, T_y, T_loc, T_scale, T_shape>
85 log_1p_y_over_lambda(N);
86
87 for (size_t i = 0; i < N; i++) {
88 const T_partials_return y_dbl = value_of(y_vec[i]);
89 const T_partials_return mu_dbl = value_of(mu_vec[i]);
90 const T_partials_return lambda_dbl = value_of(lambda_vec[i]);
91 const T_partials_return alpha_dbl = value_of(alpha_vec[i]);
92 const T_partials_return temp = 1.0 + (y_dbl - mu_dbl) / lambda_dbl;
93 const T_partials_return log_temp = log(temp);
94
95 ccdf_log[i] = -alpha_dbl * log_temp;
96
97 if (contains_nonconstant_struct<T_y, T_loc, T_scale, T_shape>::value)
98 a_over_lambda_plus_y[i] = alpha_dbl / (y_dbl - mu_dbl + lambda_dbl);
99
100 if (!is_constant_struct<T_shape>::value)
101 log_1p_y_over_lambda[i] = log_temp;
102 }
103
104 // Compute vectorized CDF and its gradients
105
106 for (size_t n = 0; n < N; n++) {
107 // Pull out values
108 const T_partials_return y_dbl = value_of(y_vec[n]);
109 const T_partials_return mu_dbl = value_of(mu_vec[n]);
110 const T_partials_return lambda_dbl = value_of(lambda_vec[n]);
111
112 // Compute
113 P += ccdf_log[n];
114
115 if (!is_constant_struct<T_y>::value)
116 operands_and_partials.d_x1[n] -= a_over_lambda_plus_y[n];
117 if (!is_constant_struct<T_loc>::value)
118 operands_and_partials.d_x2[n] += a_over_lambda_plus_y[n];
119 if (!is_constant_struct<T_scale>::value)
120 operands_and_partials.d_x3[n] += a_over_lambda_plus_y[n]
121 * (y_dbl - mu_dbl) / lambda_dbl;
122 if (!is_constant_struct<T_shape>::value)
123 operands_and_partials.d_x4[n] -= log_1p_y_over_lambda[n];
124 }
125
126 return operands_and_partials.value(P);
127 }
128 }
129 }
130 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_greater_or_equal
bool check_greater_or_equal(const char *function, const char *name, const T_y &y, const T_low &low)
Return true if y is greater or equal than low.
Definition: check_greater_or_equal.hpp:84
stan::math::pareto_type_2_ccdf_log
return_type< T_y, T_loc, T_scale, T_shape >::type pareto_type_2_ccdf_log(const T_y &y, const T_loc &mu, const T_scale &lambda, const T_shape &alpha)
Definition: pareto_type_2_ccdf_log.hpp:26
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
contains_nonconstant_struct.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::contains_nonconstant_struct
Definition: contains_nonconstant_struct.hpp:14
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
check_greater_or_equal.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::OperandsAndPartials::d_x4
VectorView< T_return_type, false, true > d_x4
Definition: OperandsAndPartials.hpp:42

[ Stan Home Page ] © 2011–2016, Stan Development Team.